﻿#include "editor.h"
#include "bridge.h"
#include "../common/sources/system/cgraphics2d.h"
#include "tools.h"
#include "rectangles.h"
#include <QDebug>

CEditor  editor;

//---------------------------------------------------------------------------------------------
CEditor::CEditor()
{
    mUndoStack = new QUndoStack();
    mSelectStart.x = -1;
}

//---------------------------------------------------------------------------------------------
// trouve le coté du mur pour une position relative à une case
EWallSide   CEditor::getWallSide(CVec2 localPos)
{
    bool side1 = (localPos.y < localPos.x);
    bool side2 = (localPos.y < TILE_SIZE - 1 - localPos.x);

    if (side1 == true && side2 == true)
        return eWallSideUp;

    if (side1 == true && side2 == false)
        return eWallSideRight;

    if (side1 == false && side2 == true)
        return eWallSideLeft;

    return eWallSideDown;
}

//---------------------------------------------------------------------------------------------
// trouve le coté du mur pour la position de la souris dans la map
EWallSide   CEditor::getWallSideAbs(CVec2 pos)
{
    return getWallSide(pos % TILE_SIZE);
}

//---------------------------------------------------------------------------------------------
CRect CEditor::getSelectionRect()
{
    CVec2   startPos = mSelectStart / TILE_SIZE;
    CVec2   endPos = mSelectEnd / TILE_SIZE;
    CRect   rect;

    // gère les cas où on ne trace pas le rectangle de haut/gauche vers bas/droite
    rect.tl.x = MIN(startPos.x, endPos.x);
    rect.tl.y = MIN(startPos.y, endPos.y);
    rect.br.x = MAX(startPos.x, endPos.x);
    rect.br.y = MAX(startPos.y, endPos.y);
    return rect;
}


//---------------------------------------------------------------------------------------------
// returns the string that tells the position of the cursor in the map
QString CEditor::getPositionString(CVec2 mousePos, ETabIndex tabIndex)
{
    CVec2 tilePos = mousePos / TILE_SIZE;
    static char tempString[256];
    tempString[0] = 0;

    if (tabIndex == eTabTiles)
    {
        sprintf(tempString, "(%d,%d)", tilePos.x, tilePos.y);
    }
    else if (tabIndex == eTabWalls)
    {
        EWallSide   side = getWallSideAbs(mousePos);

        switch (side)
        {
            case eWallSideUp:
                sprintf(tempString, "(%d,%d) up", tilePos.x, tilePos.y);
            break;

            case eWallSideDown:
                sprintf(tempString, "(%d,%d) down", tilePos.x, tilePos.y);
            break;

            case eWallSideLeft:
                sprintf(tempString, "(%d,%d) left", tilePos.x, tilePos.y);
            break;

            case eWallSideRight:
                sprintf(tempString, "(%d,%d) right", tilePos.x, tilePos.y);
            break;

            default:
            break;
        }
    }
    else if (tabIndex == eTabObjects)
    {
        CVec2   objPos = (mousePos % TILE_SIZE) / (TILE_SIZE / 2);
        sprintf(tempString, "(%d,%d) (%d,%d)", tilePos.x, tilePos.y, objPos.x, objPos.y);
    }

    return QString(tempString);
}

//---------------------------------------------------------------------------------------------
void   CEditor::hideSelectionInfos()
{
    bridge.set_tileSelScrollVis(false);
    bridge.set_wallSelScrollVis(false);
    bridge.set_objectSelScrollVis(false);
    bridge.set_rectSelScrollVis(false);
}

//---------------------------------------------------------------------------------------------
QImage CEditor::displayMap(CMap& map, CVec2 mousePos, ETabIndex tabIndex)
{
    QImage image(map.mSize.x * TILE_SIZE,
                 map.mSize.y * TILE_SIZE,
                 QImage::Format_RGBA8888);

    image.fill(QColor(0, 0, 0));

    QImage  tileGfx("tiles/Tile.png");
    QImage  wallGfx("tiles/Wall.png");

    if (map.isLoaded() == true)
    {
        // dessine les cases
        for (int y = 0; y < map.mSize.y; ++y)
            for (int x = 0; x < map.mSize.x; ++x)
            {
                CVec2   pos(x, y);
                CVec2   pos2 = pos * TILE_SIZE;

                if (map.getTile(pos)->mType != 0)
                    graph2D.drawImage(&image, pos2, tileGfx);
            }

        // dessine les murs
        for (int y = 0; y < map.mSize.y; ++y)
            for (int x = 0; x < map.mSize.x; ++x)
            {
                CVec2   pos(x, y);
                CVec2   pos2 = pos * TILE_SIZE;
                CTile*  tile = map.getTile(pos);

                if (tile->mWalls[eWallSideUp].mType != 0)
                    graph2D.drawImage(&image, pos2, wallGfx);

                if (tile->mWalls[eWallSideDown].mType != 0)
                    graph2D.drawImage(&image, pos2, wallGfx, 180);

                if (tile->mWalls[eWallSideLeft].mType != 0)
                    graph2D.drawImage(&image, pos2, wallGfx, -90);

                if (tile->mWalls[eWallSideRight].mType != 0)
                    graph2D.drawImage(&image, pos2, wallGfx, 90);
            }
    }

    rectangles.drawSelection(&image, tabIndex);
    rectangles.drawPasteSelection(&image);
    rectangles.drawCursor(&image, mousePos, tabIndex);

    return image;
}

//---------------------------------------------------------------------------------------------
void    CEditor::useTool()
{
    switch (bridge.mTool)
    {
    case eQMLToolDraw:
        toolDraw();
        break;

    case eQMLToolSelect:
        toolSelect();
        break;

    case eQMLToolPaste:
        toolPaste();
        break;
    }
}

//---------------------------------------------------------------------------------------------
void    CEditor::toolDraw()
{
    if (bridge.rightPressed == true || bridge.leftPressed == true)
    {
        if (bridge.mTabIndex == eTabTiles)
        {
            // cases
            drawTileTool*   tool = new drawTileTool();
            int type = 0;

            if (bridge.rightPressed == true)
                type = 0;
            else if (bridge.leftPressed == true)
                type = 1;

            if (tool->init(bridge.mMousePos, type) == true)
            {
                unselect();
                mUndoStack->push(tool);
            }
            else
                delete tool;
        }
        else if (bridge.mTabIndex == eTabWalls)
        {
            // murs
            drawWallTool*   tool = new drawWallTool();
            int type = 0;

            if (bridge.rightPressed == true)
                type = 0;
            else if (bridge.leftPressed == true)
                type = 1;

            if (tool->init(bridge.mMousePos, type) == true)
            {
                unselect();
                mUndoStack->push(tool);
            }
            else
                delete tool;
        }
    }
}

//---------------------------------------------------------------------------------------------
void        CEditor::copyData(CRect rect)
{
    CVec2   size = rect.br - rect.tl + CVec2(1, 1);
    mClipboard.copyFromMap(rect.tl, size);
}

//---------------------------------------------------------------------------------------------
void        CEditor::toolCopy()
{
    if (mSelectStart.x != -1)
    {
        CRect   rect = getSelectionRect();
        copyData(rect);
        unselect();
    }
}

//---------------------------------------------------------------------------------------------
void        CEditor::toolCut()
{
    if (mSelectStart.x != -1)
    {
        cutTool*   tool = new cutTool();
        CRect   rect = getSelectionRect();
        copyData(rect);

        if (tool->init(mSelectStart / TILE_SIZE, bridge.mTabIndex) == true)
            mUndoStack->push(tool);
        else
            delete tool;
    }
    unselect();
}

//---------------------------------------------------------------------------------------------
void        CEditor::toolPaste()
{
    if (bridge.leftPressing == true)
    {
        unselect();
        pasteTool*   tool = new pasteTool();

        if (tool->init(bridge.mMousePos / TILE_SIZE, bridge.mTabIndex) == true)
            mUndoStack->push(tool);
        else
            delete tool;
    }
}

//---------------------------------------------------------------------------------------------
void    CEditor::unselect()
{
    mSelectStart.x = -1;
    bridge.updateQMLImage();
}

//---------------------------------------------------------------------------------------------
void    CEditor::toolSelect()
{
    if (bridge.leftPressed == true)
    {
        mSelectEnd = bridge.mMousePos;

        if (bridge.leftPressing == true)
            mSelectStart = mSelectEnd;

        bridge.updateQMLImage();
    }
}

//---------------------------------------------------------------------------------------------
void    CEditor::initLists()
{
    bridge.tilesList.append("0 - Vide");
    bridge.tilesList.append("1 - Sol");

    bridge.wallsList.append("0 - Rien");
    bridge.wallsList.append("1 - Simple");
}

